
#!/usr/bin/env python
# -*- coding: utf-8 -*-

import maya.cmds as cmds
import hmScripts.UI.hmGradientControlFunctions as gradFunct
import hmScripts.UI.hmUiCommonUtils as uiUtil

# Creation modules
import hmScripts.Main.hmSelectionHandling as hmSelect
import hmScripts.Main.hmNodeConnector as hmNodesConnection




class BlendOptionsUI():
	def __init__(self):

		#Tooltips
		self.tooltipArray = {
		'Blend1' : 'Create a blending from 2 border edges and an optional curve',
		'Blend2' : 'Match different point count',
		'Blend3' : 'Force continuity with original meshes',

		'Corner Bias' : 'Consider a corner the angles over this degree',
		'Sections Spans' : 'Set the number of spans for the blending strips.'
		}

		#============================= UI DEFAULT ATTRIBUTES =========================================
		#We define here the default values for the options

		self.hmOpt_defaultCornerBias = 0 # prevederei di inserirlo in futuro!
		self.hmOpt_defaultSections = 5
		

		
		self.hmOpt_defaultStrategy = 3
		
		self.hmOpt_defaultSmoothProfile = 1
		# self.hmOpt_defaultFillerRefinment = 0 #Don't know If I'll use it
		self.hmOpt_defaultWeightA = 0.5
		self.hmOpt_defaultWeightB = 0.5
		self.hmOpt_defaultActiveRampedFiller = 0
		self.hmOpt_defaultProfileCurve = [u'0.5,0,2', u'0.5,1,2']
		self.hmOpt_defaultInterpolation = 4
		
		self.hmOpt_defaultRampChord = 0.1
		self.hmOpt_defaultRampOffset = -0.5
		self.hmOpt_defaultRampScale = 1
		self.hmOpt_defaultRampMode = 1  
		
		
		self.hmOpt_defaultComputeNormals = 1
		self.hmOpt_defaultContinuity = 1



		#============================= UI ATTRIBUTES =========================================
		#We store the values in these list with 2 entry: the first one is the string for the name, and the second one is the value

		self.hmOpt_SectionsVal = ['hmOpt_SectionsVal_FILLER', self.hmOpt_defaultSections]
		
		
		self.hmOpt_StrategyVal = ['hmOpt_MS_StrategyVal_FILLER', self.hmOpt_defaultStrategy]
		
		self.hmOpt_SmoothProfileVal = ['hmOpt_MS_SmoothProfileVal_FILLER', self.hmOpt_defaultSmoothProfile]
		self.hmOpt_WeightAVal = ['hmOpt_WeightAVal_FILLER', self.hmOpt_defaultWeightA]
		self.hmOpt_WeightBVal = ['hmOpt_WeightBVal_FILLER', self.hmOpt_defaultWeightB]
		self.hmOpt_ActiveRampedFillerVal  = ['self.hmOpt_ActiveRampedFillerVal_FILLER', self.hmOpt_defaultActiveRampedFiller]
		self.hmOpt_ProfileCurveVal = ['hmOpt_ProfileCurveVal_FILLER', self.hmOpt_defaultProfileCurve]
		self.hmOpt_InterpolationVal = ['hmOpt_InterpolationVal_FILLER', self.hmOpt_defaultInterpolation]
		
		self.hmOpt_RampChordVal = ['self.hmOpt_RampChordVal_FILLER', self.hmOpt_defaultRampChord]
		self.hmOpt_RampOffsetVal = ['self.hmOpt_RampOffsetVal_FILLER', self.hmOpt_defaultRampOffset]
		self.hmOpt_RampScaleVal = ['self.hmOpt_RampScaleVal_FILLER', self.hmOpt_defaultRampScale]
		self.hmOpt_RampModeVal = ['self.hmOpt_RampModeVal_FILLER', self.hmOpt_defaultRampMode]
		
		
		
		
		# self.hmOpt_AutoMergeVal = ['hmOpt_AutoMergeVal',self.hmOpt_defaultAutoMerge]
		# self.hmOpt_ToleranceVal = ['hmOpt_ToleranceVal', self.hmOpt_defaultTolerance]
		self.hmOpt_ComputeNormalsVal = ['hmOpt_ComputeNormalsVal_FILLER', self.hmOpt_defaultComputeNormals]
		self.hmOpt_ContinuityVal = ['hmOpt_ContinuityVal_FILLER', self.hmOpt_defaultComputeNormals]
		
		#============================= GRADIENT FUNCTIONS =========================================
		# =========== Gradient functions class, handle the gradient edits
		self.gFunct = gradFunct.GradientControl( self.hmOpt_ProfileCurveVal, False )


		#============================= OPTIONS VAR =========================================
		self.optionsList = [ 
		self.hmOpt_SectionsVal, self.hmOpt_StrategyVal, self.hmOpt_SmoothProfileVal, self.hmOpt_WeightAVal, self.hmOpt_WeightBVal, self.hmOpt_ActiveRampedFillerVal,
		self.hmOpt_ProfileCurveVal, self.hmOpt_InterpolationVal, self.hmOpt_RampChordVal, self.hmOpt_RampOffsetVal, self.hmOpt_RampScaleVal, self.hmOpt_RampModeVal
		]
	
		# #For every option we check if an option var is defined, if yes we sobstitute the value with the default ones
		uiUtil.optionVarInit( self.optionsList ) 


	def createUiElements(self):
		'''
		This function implements the shared UI elements for mesh setup.
		We will use the same options both on the hmDockUi and from the floating panels options, for this reason we should avoid name conflicts of the widgets...
		'''
		# cmds.separator( style='in', height = 1 )


		#================================ RIDONDANTE DA HM SETUP ========================================================
		#================== Options
		self.isGlobSections = cmds.intSliderGrp( label ='Sections Spans', field=True, columnWidth= ([2, 50], [3,100]), value = self.hmOpt_SectionsVal[1], min=2, ann =  self.tooltipArray['Sections Spans'] )
				
		
		
		# #======= Blend Strategy
		self.omStrategy = cmds.optionMenu(  label = 'Blend Strategy') 
		cmds.menuItem( label = 'Standard', data = 7 )
		cmds.menuItem( label = 'Equidistant', data = 4)
		cmds.menuItem( label = 'Experimental', data = 10 )

		# #Set the value
		cmds.optionMenu( self.omStrategy, edit = True, select = self.hmOpt_StrategyVal[1] )
		
		# #====== Profile Interpolation
		cmds.separator( height = 1, style='in' )
		self.cbContinuity = cmds.checkBox( label = 'Continuity', v = self.hmOpt_ContinuityVal[1] )
		self.cbInterpProfile = cmds.checkBox( label = 'Interpolate Blend', value = self.hmOpt_SmoothProfileVal [1], changeCommand = self.setWeightsStatus )
		self.fsWeightA = cmds.floatSliderGrp(label='weight A', field=True, columnWidth= ([2, 50], [3,100]), v=self.hmOpt_WeightAVal[1], min=0, max=1)
		self.fsWeightB = cmds.floatSliderGrp(label='weight B', field=True, columnWidth= ([2, 50], [3,100]), v=self.hmOpt_WeightBVal[1], min=0, max=1)
		# # Set the visibility of weights
		self.setWeightsStatus()
		
		# # Create the ramp check box
		self.cbActiveRamp = cmds.checkBox( label = 'Use Ramp', value = self.hmOpt_ActiveRampedFillerVal[1], onc = self.enableProfileControl,  ofc = self.disableProfileControl)

		
	
		#============================================================================================================
		#====== Gradient Layout ====== 
		self.gradFrame = cmds.frameLayout( label = 'Ramp Controls', visible=self.hmOpt_ActiveRampedFillerVal[1])
		
		# #Build controls...
		self.cbRampMode = cmds.checkBox( label = 'Parallel Move', value = self.hmOpt_RampModeVal[1], ann = 'empty' )
		self.fsRampChord = cmds.floatSliderGrp(label='Ramp Chord Resolution', field=True, columnWidth= ([2, 50], [3,100]), v=self.hmOpt_RampChordVal[1], min=0.001, max=1, precision = 4)
		self.fsRampOffset = cmds.floatSliderGrp(label='Ramp Offset', field=True, columnWidth= ([2, 50], [3,100]), v=self.hmOpt_RampOffsetVal[1], min=-10, max=10, precision = 3)
		self.fsRampScale = cmds.floatSliderGrp(label='Ramp Scale', field=True, columnWidth= ([2, 50], [3,100]), v=self.hmOpt_RampScaleVal[1], min=-10, max=10, precision = 3)
		
		self.gradLayout = cmds.rowLayout(nc=3, columnWidth3=[80,90, 5000], columnAttach3 = ['both', 'both', 'left'], columnOffset3 = [3,3,3], parent = self.gradFrame)
		cmds.columnLayout(adjustableColumn = True, rs=8, cal='right')
		cmds.text('Interpolation:')
		cmds.text('Position:')
		cmds.text('Value:')
		cmds.setParent('..') #Change the parent again to row
		
		self.gradientAttrParent = cmds.columnLayout(adjustableColumn = True, rs=3)
		self.omInterp = cmds.optionMenu( cc = self.gFunct.cangeCurrentPoint )
		self.ffInterpPos = cmds.floatField( value = 0, precision = 3, ec = self.gFunct.cangeCurrentPoint, cc = self.gFunct.cangeCurrentPoint )
		self.ffInterpVal  = cmds.floatField( value = 0, precision = 3, ec = self.gFunct.cangeCurrentPoint, cc = self.gFunct.cangeCurrentPoint )
		cmds.setParent('..') #Change the parent again to row
		
		cmds.menuItem( label = 'Linear' )
		cmds.menuItem( label = 'Smooth' )
		cmds.menuItem( label = 'Spline' )
		cmds.menuItem( label = 'None' )		
		

		#===== Third column, in the editing mode it should be probably changed with the gradient control with attr	
		cmds.columnLayout(adjustableColumn = True, rs=3)
		self.gradientParent = cmds.rowLayout( nc=2, columnWidth2=[100, 500])
			
		self.gcProfileCurve = cmds.gradientControlNoAttr( 'falloffCurve', h=90, w = 200,  optionVar=self.hmOpt_ProfileCurveVal[0], currentKeyChanged = self.gFunct.updateGradValuesUi, dragCommand = self.gFunct.updateGradValuesUi)
		self.editGradButton = cmds.button(label= '>', h=90, w= 20, visible = True )

		# =========== Gradient functions class, set the controls
		self.gFunct.setControls( self.gcProfileCurve, self.omInterp, self.ffInterpPos, self.ffInterpVal )


		cmds.setParent('..') #Change the parent again to row		
		cmds.setParent('..') #Change the parent again to row
		#============================================================================================================
		#============================================================================================================

		cmds.setParent( '..' )

		cmds.rowLayout( nc=2, columnWidth2=[100, 500])
		cmds.text('Curve presets:')
		cmds.rowLayout(nc=5)
		self.button1 = cmds.symbolButton(image= 'gradientControl_icon_01.png', c=self.gFunct.setGradientControlTo1 )
		self.button2 = cmds.symbolButton(image= 'gradientControl_icon_02.png', c=self.gFunct.setGradientControlTo2 )
		self.button3 = cmds.symbolButton(image= 'gradientControl_icon_03.png', c=self.gFunct.setGradientControlTo3 )
		self.button4 = cmds.symbolButton(image= 'gradientControl_icon_04.png', c=self.gFunct.setGradientControlTo4 )
		self.button4 = cmds.symbolButton(image= 'gradientControl_icon_05.png', c=self.gFunct.setGradientControlTo5 )

		#================================ RIDONDANTE DA HM SETUP ========================================================
				
		#Reset the parent for the next controls
		cmds.setParent( '..' )
		cmds.setParent( '..' )
		cmds.setParent( '..' )
		cmds.setParent( '..' )
		cmds.setParent( '..' )


	#================================ COMMANDS ========================================================
	def performCommand(self, *Args ):
		'''
		Perform the command
		'''		
		#Init the sel handling
		selection = hmSelect.HmSelectionHandling()
		# Extract the curves, if there are some..
		curves = selection.extract( 'nurbsCurve' )

		#Set custom messages
		selection.setErrorMessageIfZero('Please select 2 edge loops and an one optional middle curve to perform the command.')
		selection.setErrorIfSmallThanMin('Please select at least 2 edge loops to operate on.')
		selection.setErrorMessageIfOutLimit('Please select only 2 edge loop and no more than 1 middle curve.')
		#Validate the selection and put the result on a var 
		components = selection.ignoreComponents() #<<<<<< here check that the components are of the valid type

		objects = selection.validate(1,2,'mesh')
		components = cmds.ls( orderedSelection=True )

		# =========================== 2 MESHES CASE ==================================
		# Validate now the component selection
		if len(objects) == 2:
			sideA = []
			sideB = []

			for component in components:
			    #index = int( component.split('[')[1].split(']')[0] ) # This returns only the index, but we want e[xxx]
			    index = component.split('.')[1]
			    if objects[0] in component:
			        sideA.append( index )
			    if objects[1] in component:
			        sideB.append( index )

			print 'sideA', sideA
			print 'sideB', sideB
		# =========================== 1 MESHES CASE ==================================
		if len(objects) == 1:
			cmds.error('One mesh setup not supported yet.')


		# check that they are all edges

		# create 2 list based on connectivity <<<<< this has to be in the connection class?

	
		#==== Get the interface values
		self.getValuesFromInterface()

		
		# #==== Create the node network
		# #Be careful that we are converting the strategy with our utility dictionary
		# hmSetup = hmNodesConnection.HmOperationConnections()
		
		# hmSetup.hmCompoundSetup( sel[0], sel[1], self.hmopt_operation_val[1], self.hmopt_smooth_val_a[1], self.hmopt_smooth_val_b[1], self.hmopt_offset_val[1], self.hmopt_offset_val[1], self.hmopt_corner_bias_val[1],
		# 									self.hmopt_geo_bias_val[1], self.hmopt_sections_val[1], self.hmopt_refine_val[1]-1, self.hmopt_chord_dist_val[1],  gVar.uiFillStrategy[ self.hmOpt_StrategyVal[1] ],
		# 									self.hmopt_smooth_profile_val[1], self.hmopt_weight_a_val[1], self.hmopt_weight_b_val[1], self.hmopt_active_ramped_filler_val[1], self.hmopt_profile_curve_val[1],
		# 									self.hmopt_ramp_chord_val[1], self.hmopt_ramp_offset_val[1], self.hmopt_ramp_scale_val[1],  self.hmopt_ramp_mode_val[1], self.hmopt_bind_bias_val[1], self.hmopt_remove_curl_val[1],
		# 									self.hmopt_compute_normals_val[1], self.hmopt_offset_type[1]) #We will use the arg 0 as shader


		#Set the handler options
		# self.setOptions()
		blendClass = hmNodesConnection.HmBlendConnections()
		
		#======== Perform command
		blendClass.performMeshMeshBlend(objects[0], objects[1], sideA, sideB)
		

		

	def saveOptionsValues(self):
		'''
		This method stores the current options in the optionVar 
		'''
		#Get the interface values
		self.getValuesFromInterface()
		#Save them
		uiUtil.optionVarSave( self.optionsList )



	def resetOptionsValues(self):
		'''
		This method reset the options with the defualt values
		'''

		#Very important TRICK, let the gradient control forget about its option
		cmds.gradientControlNoAttr( self.gcProfileCurve, e=True, optionVar='' )
		
		
		#Reset the ui values
		cmds.intSliderGrp( self.isGlobSections, e=True, v= self.hmOpt_defaultSections )
		
		#Advanced				
		cmds.optionMenu( self.omStrategy, edit=True, select=self.hmOpt_defaultStrategy )
		
		cmds.checkBox( self.cbInterpProfile, e=True, v = self.hmOpt_defaultSmoothProfile )
		cmds.floatSliderGrp( self.fsWeightA, e=True, v = self.hmOpt_defaultWeightA )
		cmds.floatSliderGrp( self.fsWeightB, e=True, v = self.hmOpt_defaultWeightB )
		cmds.checkBox( self.cbActiveRamp,  e=True, v =self.hmOpt_defaultActiveRampedFiller )
		#Hide the ramp controls, since the default has the use ramp unactive
		self.disableProfileControl()
		

		#Set the gradient control to the preset n.1
		self.gFunct.setGradientControlTo1
		
		cmds.checkBox( self.cbRampMode, e=True, v= self.hmOpt_defaultRampMode )
		cmds.floatSliderGrp( self.fsRampChord, e=True, v=self.hmOpt_defaultRampChord )
		cmds.floatSliderGrp( self.fsRampOffset, e=True, v=self.hmOpt_defaultRampOffset)
		cmds.floatSliderGrp( self.fsRampScale, e=True, v=self.hmOpt_defaultRampScale)
		
		cmds.optionMenu(self.omInterp, e=True, select=self.hmOpt_defaultInterpolation )





	#================================ INTERFACE UPDATES ========================================================


	#================================ RIDONDANTE DA HM SETUP ========================================================
	
	def controlRampStatus(self):
		'''
		Since opening and closing the frame layout ignore the status of the control
		making it always visible, we have to check it again 
		'''
		#Get the current check box value...
		isOnOrOff = cmds.checkBox( self.cbActiveRamp, q=True, v = True )
		#But if its a par changer, get the attribute...
		if self.asParameterChanger:
			isOnOrOff = cmds.getAttr( self.nodeUseRamp )
		
		# Now we know if when opening the layout the control must be visible or not
		if isOnOrOff:
			self.enableProfileControl()
		else:
			self.disableProfileControl()


	def setWeightsStatus(self, *Args):
		valOfInterpolation = cmds.checkBox( self.cbInterpProfile, q=True, v = True )
		
		if valOfInterpolation == 0:
			cmds.floatSliderGrp( self.fsWeightA, e=True, vis = False )
			cmds.floatSliderGrp( self.fsWeightB, e=True, vis = False )
		else:
			cmds.floatSliderGrp( self.fsWeightA, e=True, vis = True )
			cmds.floatSliderGrp( self.fsWeightB, e=True, vis = True )


	def enableProfileControl(self, *Args):
		#Make the ramp controls visible
		cmds.frameLayout( self.gradFrame, edit=True, visible=True )
		#Disable the sections spans
		cmds.intSliderGrp( self.isGlobSections,  edit=True, en=False)
	def disableProfileControl(self, *Args):
		#Make the ramp controls invisible
		cmds.frameLayout( self.gradFrame, edit=True, visible=False )
		#Enable the sections spans
		cmds.intSliderGrp( self.isGlobSections,  edit=True, en=True)

	#================================ RIDONDANTE DA HM SETUP ========================================================



	#================================ UTILITY ========================================================

	def getValuesFromInterface(self, *Args):
		'''
		This method store in the main variables the values of the interface
		'''
		print 'SONO NEL GET!!!!!!'
		self.hmOpt_SectionsVal[1] = cmds.intSliderGrp( self.isGlobSections, q=True, v=True)
		
		#Advanced
		self.hmOpt_StrategyVal[1] =  cmds.optionMenu( self.omStrategy, q=True, select=True )

			
		
		self.hmOpt_SmoothProfileVal[1] = cmds.checkBox( self.cbInterpProfile, q=True, v = True )

		self.hmOpt_WeightAVal[1] = cmds.floatSliderGrp( self.fsWeightA, q=True, v = True )
		self.hmOpt_WeightBVal[1] = cmds.floatSliderGrp( self.fsWeightB, q=True, v = True )


		self.hmOpt_ProfileCurveVal[1] = cmds.optionVar( q=self.hmOpt_ProfileCurveVal[0])

		print 'il valore della prifile curve ', self.hmOpt_ProfileCurveVal[1]

		self.hmOpt_InterpolationVal[1] = cmds.optionMenu(self.omInterp, q=True, select=True )

		self.hmOpt_RampChordVal[1] = cmds.floatSliderGrp( self.fsRampChord, q=True, v=True )
		self.hmOpt_RampOffsetVal[1] =  cmds.floatSliderGrp( self.fsRampOffset, q=True, v=True)
		self.hmOpt_RampScaleVal[1] =  cmds.floatSliderGrp( self.fsRampScale, q=True, v=True)
		self.hmOpt_RampModeVal[1] =  cmds.checkBox(self.cbRampMode, q=True, v=True )
		
		

		


	def getMainTooltip(self):
		'''
		This method return the main tooltip hint for adding it to the main button
		'''
		return self.tooltipArray['Blend1']




