import maya.cmds as cmds
import maya.mel as mel
import hmScripts.hm_global_variables as gVar
import hmScripts.hm_global_variables as nc
import hmScripts.UI.hmUiCommonUtils as uiUtil
import hmScripts.UI.hmGradientControlFunctions as gradFunct
import hmScripts.Main.hmPerform_new_operation as command
from hmScripts.hm_global_variables import language
from collections import OrderedDict

tooltip = language.values['new_operation_win']['tooltips']
labels = language.values['new_operation_win']


class HardMeshOperationOptionsUI(object):

    # This mapping associate the label of the shader to the real name,
    # it is used for supporting the chinese version of shader names
    #
    SHADERS_NAMES = OrderedDict([(labels['default_shader'], nc.default_shader),
                                 (labels['shader_from_A'], nc.shader_from_A),
                                 (labels['shader_from_B'], nc.shader_from_B),
                                 (labels['hm_dark_shader'], nc.hm_dark_shader),
                                 (labels['hm_light_shader'], nc.hm_light_shader),
                                 (labels['custom_sh'], nc.custom_shader)])

    def __init__(self, as_parameter_changer=False):
        self.connected_compound = None
        self.connected_order_to_split = None
        self.connected_offset_node = None
        self.force_operation = 0

        # -------------------------------------------- PARAMETER CHANGER --------------------------------------------- #
        self.as_parameter_changer = as_parameter_changer
        self.connected_compound = None
        self.connected_order_to_split = None
        self.opt_offset_val_a = None
        self.opt_offset_val_b = None

        # hmBaseObjects
        self.mesh_A = None
        self.meshes_B = None

        # ------------------------------------------ UI DEFAULT ATTRIBUTES ------------------------------------------- #
        self.hmopt_default_offset_type = 1

        self.hmopt_default_operation = 1
        self.hmopt_default_smooth_a = 1
        self.hmopt_default_smooth_b = 1
        self.hmopt_default_offset_val = 0.1
        self.hmopt_default_corner_bias = 0
        self.hmopt_default_geo_bias = 0.0
        self.hmopt_default_sections = 5

        self.hmopt_default_refine = 2
        self.hmopt_default_chord_dist = 0.01

        self.hmopt_default_smooth_profile = 1
        self.hmopt_default_weight_a = 0.5
        self.hmopt_default_weight_b = 0.5
        self.hmopt_default_active_ramped_filler = 0
        self.hmopt_default_profile_curve = [u'0.5,0,2', u'0.5,1,2']
        self.hmopt_default_interpolation = 4

        self.hmopt_default_ramp_chord = 0.1
        self.hmopt_default_ramp_offset = -0.5
        self.hmopt_default_ramp_scale = -1
        self.hmopt_default_ramp_mode = 1

        # Order to split related
        self.hmopt_default_bind_bias = 0.01
        # self.hmOpt_defaultSequencer = 0
        self.hmopt_default_sequencer_tolerance = 0.001
        self.hmopt_default_remove_curl = 0

        # self.hmOpt_defaultAutoMerge = 1
        # self.hmOpt_defaultTolerance = 0.001
        self.hmopt_default_compute_normals = 1

        # Store the default shader id
        self.hmopt_default_output_shader = self.SHADERS_NAMES.values()[1]

        # ------------------------------------------- UI ATTRIBUTES -------------------------------------------------- #
        # We store the values in these list with 2 entry:
        # the first one is the string for the name, and the second one is the value

        self.hmopt_offset_type = ['hmopt_offset_type', self.hmopt_default_offset_type]

        self.hmopt_operation_val = ['hmopt_operation_val', self.hmopt_default_operation]
        self.hmopt_smooth_val_a = ['hmopt_smooth_val_a ', self.hmopt_default_smooth_a]
        self.hmopt_smooth_val_b = ['hmopt_smooth_val_b ', self.hmopt_default_smooth_b]
        self.hmopt_offset_val = ['hmopt_offset_val', self.hmopt_default_offset_val]
        self.hmopt_corner_bias_val = ['hmopt_corner_bias_val', self.hmopt_default_corner_bias]
        self.hmopt_geo_bias_val = ['hmopt_geo_bias_val', self.hmopt_default_geo_bias]
        self.hmopt_sections_val = ['hmopt_sections_val', self.hmopt_default_sections]

        self.hmopt_refine_val = ['hmopt_refine_val', self.hmopt_default_refine]
        self.hmopt_chord_dist_val = ['hmOpt_MS_ChordDistVal', self.hmopt_default_chord_dist]

        self.hmopt_smooth_profile_val = ['hmopt_smooth_profile_val', self.hmopt_default_smooth_profile]
        # self.hmOpt_FillerRefinmentVal = ['hmOpt_FillerRefinmentVal', self.hmOpt_defaultFillerRefinment]
        self.hmopt_weight_a_val = ['hmopt_weight_a_val', self.hmopt_default_weight_a]
        self.hmopt_weight_b_val = ['hmopt_weight_b_val', self.hmopt_default_weight_b]
        self.hmopt_active_ramped_filler_val = ['self.hmopt_active_ramped_filler_val', self.hmopt_default_active_ramped_filler]
        self.hmopt_profile_curve_val = ['hmopt_profile_curve_val', self.hmopt_default_profile_curve]
        self.hmopt_interpolation_val = ['hmopt_interpolation_val', self.hmopt_default_interpolation]

        self.hmopt_ramp_chord_val = ['hmopt_ramp_chord_val', self.hmopt_default_ramp_chord]
        self.hmopt_ramp_offset_val = ['hmopt_ramp_offset_val', self.hmopt_default_ramp_offset]
        self.hmopt_ramp_scale_val = ['hmopt_ramp_scale_val', self.hmopt_default_ramp_scale]
        self.hmopt_ramp_mode_val = ['hmopt_ramp_mode_val', self.hmopt_default_ramp_mode]

        # Order to split related
        self.hmopt_bind_bias_val = ['self.hmopt_bind_bias_val', self.hmopt_default_bind_bias]
        # self.hmOpt_SequencerVal = ['self.hmOpt_SequencerVal', self.hmOpt_defaultSequencer]
        # self.hmOpt_SequencerToleranceVal = ['self.hmOpt_SequencerToleranceVal', self.hmopt_default_sequencer_tolerance]
        self.hmopt_remove_curl_val = ['self.hmopt_remove_curl_val', self.hmopt_default_remove_curl]

        # self.hmOpt_AutoMergeVal = ['hmOpt_AutoMergeVal',self.hmOpt_defaultAutoMerge]
        # self.hmOpt_ToleranceVal = ['hmOpt_ToleranceVal', self.hmOpt_defaultTolerance]
        self.hmopt_compute_normals_val = ['hmopt_compute_normals_val', self.hmopt_default_compute_normals]

        self.hmopt_out_shader_val = ['hmopt_out_shader_val', self.hmopt_default_output_shader]

        self.current_custom_shader = None
        # -------------------------------------------- GRADIENT FUNCTIONS -------------------------------------------- #
        # =========== Gradient functions class, handle the gradient edits
        self.gFunct = gradFunct.GradientControl(self.hmopt_profile_curve_val, self.as_parameter_changer)

        # ----------------------------------------------- OPTIONS VAR ------------------------------------------------ #
        if self.as_parameter_changer is False:
            self.optionsList = [
                self.hmopt_operation_val, self.hmopt_smooth_val_a, self.hmopt_smooth_val_b, self.hmopt_offset_val,
                self.hmopt_corner_bias_val, self.hmopt_geo_bias_val, self.hmopt_sections_val,
                self.hmopt_refine_val, self.hmopt_chord_dist_val, self.hmopt_smooth_profile_val, self.hmopt_weight_a_val,
                self.hmopt_weight_b_val, self.hmopt_active_ramped_filler_val,
                self.hmopt_profile_curve_val, self.hmopt_interpolation_val, self.hmopt_ramp_chord_val,
                self.hmopt_ramp_offset_val, self.hmopt_ramp_scale_val, self.hmopt_ramp_mode_val,
                self.hmopt_bind_bias_val, self.hmopt_remove_curl_val, self.hmopt_compute_normals_val, self.hmopt_out_shader_val
            ]

            # #For every option we check if an option var is defined, if yes we sobstitute the value with the default ones
            uiUtil.optionVarInit(self.optionsList)
            # uiUtil.optionVarRemove(self.optionsList)

    def setForcedAttributes(self, forceOperation):
        """
        We use this method for forcing some attributes for UI purposes with the icons on the main window.
        """
        self.force_operation = forceOperation

        # If we want to build the window with a forced operation
        if self.force_operation == 1:
            self.hmopt_operation_val = ['hmOpt_OperationValUn', 1]
        elif self.force_operation == 2:
            self.hmopt_operation_val = ['hmOpt_OperationValAB', 2]
        elif self.force_operation == 3:
            self.hmopt_operation_val = ['hmOpt_OperationValBA', 3]
        elif self.force_operation == 4:
            self.hmopt_operation_val = ['hmOpt_OperationValInt', 4]
        elif self.force_operation == 5:
            self.hmopt_operation_val = ['hmOpt_OperationValPAB', 5]
        elif self.force_operation == 6:
            self.hmopt_operation_val = ['hmOpt_OperationValPBA', 6]

        return self

    def initCompoundAttr(self, meshes_list):
        """
        THis method is called for building the attributes strings for the current compound node

        Args:
            meshes_list: list of hmBaseObjects
        """

        # Get the connected meshes
        self.mesh_A = meshes_list[0]
        self.meshes_B = meshes_list[1:]

        # #We use the attributes of the smooth directly on the meshes
        self.smooth_node_A = '%s.%s' % (self.mesh_A.get_name(), gVar.smooth_attr_name)
        self.smooth_nodes_B = ['%s.%s' % (mesh.get_name(), gVar.smooth_attr_name) for mesh in self.meshes_B]

        self.nodeCornerBias = '%s.%s' % (self.connected_compound, gVar.corner_bias)
        self.nodeGeometryBias = '%s.%s' % (self.connected_compound, gVar.geometry_bias)
        self.nodeSectionSpans = '%s.%s' % (self.connected_compound, gVar.section_spans)

        self.nodeRefineEdge = '%s.%s' % (self.connected_compound, gVar.edge_subd)
        self.nodeChordDist = '%s.%s' % (self.connected_compound, gVar.chord_dist)

        self.nodeFillStrategy = '%s.%s' % (self.connected_compound, gVar.fill_strategy)
        # self.nodeRefineFiller = '%s.%s' % (self.connectedCompound, gVar.refinement)

        self.nodeSmoothFiller = '%s.%s' % (self.connected_compound, gVar.smooth_filler)

        self.nodeWeightA = '%s.%s' % (self.connected_compound, gVar.weight_A)
        self.nodeWeightB = '%s.%s' % (self.connected_compound, gVar.weight_B)

        self.nodeUseRamp = '%s.%s' % (self.connected_compound, gVar.use_ramp)

        self.nodeRampChord = '%s.%s' % (self.connected_compound, gVar.ramp_chord_dist)
        self.nodeRampOffset = '%s.%s' % (self.connected_compound, gVar.ramp_offset)
        self.nodeRampScale = '%s.%s' % (self.connected_compound, gVar.ramp_scale)
        self.nodeRampMode = '%s.%s' % (self.connected_compound, gVar.ramp_mode)

        # Gradient options, check the gradient control with the attribute..
        self.nodeRampedFiller = '%s.%s' % (self.connected_compound, gVar.ramp_filler)

        # Static ID ramp attr
        self.nodeRampedFiller_InterpONE = '%s.%s[1].%s' % (self.connected_compound, gVar.ramp_filler, gVar.ramp_int)
        self.nodeRampedFiller_ValONE = '%s.%s[1].%s' % (self.connected_compound, gVar.ramp_filler, gVar.ramp_val)
        self.nodeRampedFiller_PosONE = '%s.%s[1].%s' % (self.connected_compound, gVar.ramp_filler, gVar.ramp_pos)

        self.nodeComputeNormals = '%s.%s' % (self.connected_compound, gVar.compute_normals)
        self.nodeCompoundScaleFactor = '%s.%s' % (self.connected_compound, gVar.scale_factor)

        # Order To split Parameters
        self.nodeBindBias = '%s.%s' % (self.connected_order_to_split, gVar.bindBias)
        self.nodeRemoveCurl = '%s.%s' % (self.connected_order_to_split, gVar.remove_curl)
        self.nodeCurveSequencer = '%s.%s' % (self.connected_order_to_split, gVar.curveSequencer)
        self.nodeSequenceBias = '%s.%s' % (self.connected_order_to_split, gVar.sequenceBias)

        # Offset node Parameters
        self.nodeOperationType = '%s.%s' % (self.connected_offset_node, gVar.boolean_operation)
        self.nodeOffsetRemoveCurl = '%s.%s' % (self.connected_offset_node, gVar.remove_curl)
        self.nodeOffsetScaleFactor = '%s.%s' % (self.connected_offset_node, gVar.scale_factor)
        self.nodeOffsetAttrNameA = '%s.%s' % (self.connected_offset_node, gVar.offset_A)
        self.nodeOffsetAttrNameB = '%s.%s' % (self.connected_offset_node, gVar.offset_B)

    def createUiElements(self):
        """
        This function implements the shared UI elements for mesh setup.
        We will use the same options both on the hmDockUi and from the floating panels options,
        for this reason we should avoid name conflicts of the widgets...
        """

        # Main Layout
        cmds.columnLayout(adj=1, rowSpacing=4)

        # Variable number of columns, 2 for creation UI, 3 for parameter changer with the "apply" button
        columns = 2

        if self.as_parameter_changer:
            columns = 3  # If parameter changer, 3 columns for apply buttton
        else:
            # ====== DEVELOPING VAR
            if gVar.develop == 1:
                columns += 1

        self.firstRow = cmds.rowLayout('first row', nc=columns)

        # ====== DEVELOPING VAR
        if gVar.develop == 1 and self.as_parameter_changer == False:
            columns += 1
            self.offTypeOperation = cmds.radioButtonGrp('rOffType', labelArray2=['hmOffset', 'mayaOffset'],
                                                        numberOfRadioButtons=2, columnWidth=([1, 80], [2, 100]),
                                                        select=self.hmopt_offset_type[1], en=True)

        self.omOperation = cmds.optionMenu(label=labels['operation'])
        cmds.menuItem(label=labels["operation_a"], data=0)
        cmds.menuItem(label=labels["operation_b"], data=1)
        cmds.menuItem(label=labels["operation_c"], data=2)
        cmds.menuItem(label=labels["operation_d"], data=3)
        cmds.menuItem(label=labels["operation_e"], data=4)
        cmds.menuItem(label=labels["operation_f"], data=5)

        # Set the value
        cmds.optionMenu(self.omOperation, edit=True, select=self.hmopt_operation_val[1])
        self.ifgSmoothAB = cmds.intFieldGrp(numberOfFields=2,
                                            label=labels['smooth'],
                                            columnWidth = [(1 ,100), (2 ,30), (3 ,30)],
                                            adj = 1,
                                            value1= self.hmopt_smooth_val_a[1],
                                            value2= self.hmopt_smooth_val_b[1],
                                            ann = tooltip['smooth_mesh'])

        # If we are forcing the operation, like the option boxes from hmWindow, lock the option menu..
        if self.force_operation != 0:
            cmds.optionMenu(self.omOperation, edit = True, en= False )

        if self.as_parameter_changer:
            self.smoothButon = cmds.button(label = labels['apply'], command = self.changeSmoothValues)

        cmds.setParent( '..' )
        cmds.separator( height = 1, style='in' )

        maxInitOffset = 2
        maxInitChord = 1
        absoluteMax = 100

        if self.as_parameter_changer:
            # Create the UI elements and leave visible only the relevant ones
            self.doubleOffsetLayout = cmds.columnLayout( adj = 1, visible = False)
            cmds.rowLayout( nc = 2, adj = 1, columnAttach2 = ['both', 'both' ] , visible = True )

            self.fsOffsetA = cmds.floatSliderGrp(label =labels['offset_a'],
                                                 field=True,
                                                 precision = 3,
                                                 columnWidth= ([2, 50], [3 ,100]),
                                                 value = self.hmopt_offset_val[1],
                                                 max = maxInitOffset,
                                                 min = 0,
                                                 fieldMaxValue =  absoluteMax)

            cmds.button(label = labels['merge_sides'], c=self.mergeOffset )
            cmds.setParent('..')
            cmds.rowLayout( nc = 2, adj = 1, columnAttach2 = ['both', 'both' ] , visible = True )

            self.fsOffsetB = cmds.floatSliderGrp(label =labels['offset_b'],
                                                 field=True,
                                                 precision = 3,
                                                 columnWidth= ([2, 50], [3 ,100]),
                                                 value = self.hmopt_offset_val[1],
                                                 max = maxInitOffset,
                                                 min = 0,
                                                 fieldMaxValue =  absoluteMax)

            cmds.button(label = labels['merge_sides'], visible = False )
            cmds.setParent('..')

            # Back to column
            cmds.setParent('..')

            self.singleOffsetLayout = cmds.rowLayout( nc = 2, adj = 1, columnAttach2 = ['both', 'both' ] , visible = True )

            # Check if we have to display one or wto slider
            self.fsOffset = cmds.floatSliderGrp(label =labels['offset'],
                                                field=True,
                                                precision = 3,
                                                columnWidth= ([2, 50], [3 ,100]),
                                                value = self.hmopt_offset_val[1],
                                                max = maxInitOffset,
                                                min = 0,
                                                fieldMaxValue =  absoluteMax)
            cmds.button(label = labels['split_sides'], c=self.splitOffset )
            cmds.setParent('..')
        # ==============================================================================
        else:
            self.fsOffset = cmds.floatSliderGrp(label =labels['offset'],
                                                field=True,
                                                precision = 3,
                                                columnWidth= ([2, 50], [3 ,100]),
                                                value = self.hmopt_offset_val[1],
                                                max = maxInitOffset,
                                                min = 0,
                                                ann = tooltip['offset'])

        # ==============================================================================
        # The visibility of corner bias is on only for the develop...
        self.fsGlobCorBias = cmds.floatSliderGrp(label =labels['corner_bias'],
                                                 field=True,
                                                 columnWidth= ([2, 50], [3 ,100]),
                                                 value = self.hmopt_corner_bias_val[1],
                                                 max=180, ann =  tooltip['corner_bias'],
                                                 en=False,
                                                 visible = gVar.develop)

        # These two are always visible
        self.fsGlobGeoBias = cmds.floatSliderGrp(label =labels['geometry_bias'],
                                                 field=True,
                                                 columnWidth= ([2, 50], [3 ,100]),
                                                 value = self.hmopt_geo_bias_val[1],
                                                 precision = 3,
                                                 max = 1,
                                                 min = 0,
                                                 ann =  tooltip['geo_bias'])

        self.isGlobSections = cmds.intSliderGrp(label =labels['sections_spans'],
                                                field=True,
                                                columnWidth= ([2, 50], [3 ,100]),
                                                value = self.hmopt_sections_val[1],
                                                min=2,
                                                ann =  tooltip['sections_spans'])


        # Create the chord dist parameter
        self.isAdaptVal = cmds.floatSliderGrp(label =labels['chord_dist'],
                                              field=True,
                                              columnWidth= ([2, 50], [3 ,100]),
                                              precision = 6,
                                              value = self.hmopt_chord_dist_val[1],
                                              max = maxInitChord,
                                              min = 0,
                                              ann = tooltip['chord_dist'],
                                              fieldMaxValue =  absoluteMax)
        # self.setRefineOptStatus()


        self.omRefEdge = cmds.optionMenuGrp( label = labels['split_refine'],
                                             changeCommand = self.setRefineOptStatus,
                                             ann =  tooltip['split_refine'] )
        cmds.menuItem( label = labels['refine_off'], data = 0 )
        cmds.menuItem( label = labels['refine_c_dist'], data = 1 )
        cmds.menuItem( label = labels['refine_strict'], data = 2 )
        cmds.optionMenuGrp(self.omRefEdge, edit = True, select = self.hmopt_refine_val[1])

        # ----------------------------------------------- BLENDING TAB ----------------------------------------------- #
        self.blendingTab = cmds.frameLayout( label = labels['blending'] ,
                                             collapsable= True ,
                                             collapse = False,
                                             expandCommand = self.controlRampStatus,
                                             mh=10,
                                             mw=10 )

        # ====== Profile Interpolation
        self.cbInterpProfile = cmds.checkBox(label = labels['interpolate_blend'],
                                             value = self.hmopt_smooth_profile_val [1],
                                             changeCommand = self.setWeightsStatus,
                                             ann = tooltip['interpolate_blend'])

        self.fsWeightA = cmds.floatSliderGrp(label=labels['weight_a'],
                                             field=True,
                                             columnWidth= ([2, 50], [3 ,100]),
                                             v=self.hmopt_weight_a_val[1],
                                             min=0,
                                             max=1,
                                             ann = tooltip['weight_a'])

        self.fsWeightB = cmds.floatSliderGrp(label=labels['weight_b'],
                                             field=True,
                                             columnWidth= ([2, 50], [3 ,100]),
                                             v=self.hmopt_weight_b_val[1],
                                             min=0,
                                             max=1,
                                             ann = tooltip['weight_b'])
        # Set the visibility of weights
        self.setWeightsStatus()

        # Create the ramp check box
        self.cbActiveRamp = cmds.checkBox(label = labels['use_ramp'],
                                          value = self.hmopt_active_ramped_filler_val[1],
                                          onc = self.enableProfileControl,
                                          ofc = self.disableProfileControl,
                                          ann = tooltip['use_ramp'])

        # ============================================================================================================
        # ====== Gradient Layout ======
        self.gradFrame = cmds.frameLayout(label = labels['ramp_controls'],
                                          visible=self.hmopt_active_ramped_filler_val[1])

        # #Build controls...

        # Visible if in develop
        if gVar.develop == 1:
            self.fsRampChord = cmds.floatSliderGrp(label=labels['ramp_chord_resolution'],
                                                   field=True,
                                                   columnWidth= ([2, 50], [3 ,100]),
                                                   v=self.hmopt_ramp_chord_val[1],
                                                   min=0.001,
                                                   max=1,
                                                   precision = 4)

            self.cbRampMode = cmds.checkBox(label = labels['parallel_move'],
                                            value = self.hmopt_ramp_mode_val[1],
                                            ann = tooltip['split2'])
        # Hidden for the current build
        else:
            self.fsRampChord = cmds.floatSliderGrp(label=labels['ramp_chord_resolution'],
                                                   field=True,
                                                   columnWidth= ([2, 50], [3 ,100]),
                                                   v=self.hmopt_ramp_chord_val[1],
                                                   min=0.001,
                                                   max=1,
                                                   precision = 4,
                                                   visible = False)

        self.fsRampOffset = cmds.floatSliderGrp(label=labels['ramp_offset'],
                                                field=True,
                                                columnWidth= ([2, 50], [3 ,100]),
                                                v=self.hmopt_ramp_offset_val[1],
                                                min=-10,
                                                max=10,
                                                precision = 3)
        self.fsRampScale = cmds.floatSliderGrp(label=labels['ramp_scale'],
                                               field=True,
                                               columnWidth= ([2, 50], [3 ,100]),
                                               v=self.hmopt_ramp_scale_val[1],
                                               min=-10,
                                               max=10,
                                               precision = 3)





        self.gradLayout = cmds.rowLayout(nc=3,
                                         columnWidth3=[80 ,90, 5000],
                                         columnAttach3 = ['both', 'both', 'left'],
                                         columnOffset3 = [3 ,3 ,3],
                                         parent = self.gradFrame)


        cmds.columnLayout(adjustableColumn = True, rs=8, cal='right')
        cmds.text(labels['interpolation'])
        cmds.text(labels['position'])
        cmds.text(labels['value'])
        cmds.setParent('..')  # Change the parent again to row

        self.gradientAttrParent = cmds.columnLayout(adjustableColumn = True, rs=3)
        self.omInterp = cmds.optionMenu( cc = self.gFunct.cangeCurrentPoint )
        self.ffInterpPos = cmds.floatField( value = 0, precision = 3, ec = self.gFunct.cangeCurrentPoint, cc = self.gFunct.cangeCurrentPoint )
        self.ffInterpVal  = cmds.floatField( value = 0, precision = 3, ec = self.gFunct.cangeCurrentPoint, cc = self.gFunct.cangeCurrentPoint )
        cmds.setParent('..')  # Change the parent again to row

        cmds.menuItem( label = labels['linear'] )
        cmds.menuItem( label = labels['smooth'] )
        cmds.menuItem( label = labels['spline'] )
        cmds.menuItem( label = labels['none'] )

        # ===== Third column, in the editing mode it should be probably changed with the gradient control with attr
        cmds.columnLayout(adjustableColumn = True, rs=3)

        self.gradientParent = cmds.rowLayout( nc=2, columnWidth2=[100, 500])
        self.gcProfileCurve = cmds.gradientControlNoAttr( 'falloffCurve',
                                                          h=90,
                                                          w = 200,
                                                          optionVar=self.hmopt_profile_curve_val[0],
                                                          currentKeyChanged = self.gFunct.updateGradValuesUi,
                                                          dragCommand = self.gFunct.updateGradValuesUi)
        self.editGradButton = cmds.button(label= '>',
                                          h=90,
                                          w= 20,
                                          c=self.editGradientAttributeInWindow,
                                          visible = self.as_parameter_changer)


        # =========== Gradient functions class, set the controls
        self.gFunct.setControls( self.gcProfileCurve, self.omInterp, self.ffInterpPos, self.ffInterpVal )

        cmds.setParent('..')
        cmds.setParent('..')


        # Change the parent to..
        cmds.setParent('..')


        cmds.rowLayout( nc=2, columnWidth2=[100, 500])
        cmds.text(labels['curve_presets'])
        cmds.rowLayout(nc=5)
        self.button1 = cmds.symbolButton(image= 'gradientControl_icon_01.png', c= self.gFunct.setGradientControlTo1 )
        self.button2 = cmds.symbolButton(image= 'gradientControl_icon_02.png', c= self.gFunct.setGradientControlTo2 )
        self.button3 = cmds.symbolButton(image= 'gradientControl_icon_03.png', c= self.gFunct.setGradientControlTo3 )
        self.button4 = cmds.symbolButton(image= 'gradientControl_icon_04.png', c= self.gFunct.setGradientControlTo4 )
        self.button4 = cmds.symbolButton(image= 'gradientControl_icon_05.png', c= self.gFunct.setGradientControlTo5 )

        # Close the blending tab
        cmds.setParent(self.blendingTab)



        # #=========================================>>>>> SHADER TAB <<<<<<<<============================================================

        if not self.as_parameter_changer:
            cmds.setParent('..')
            self.shadingTab = cmds.frameLayout( label = labels['out_mesh_shader'] ,
                                                collapsable= True ,
                                                collapse = False,
                                                mh=10,
                                                mw=10 )  # bgs=True maya 2015 non lo sopporta..


            # Reference var
            #
            shader_stored = self.hmopt_out_shader_val[1]

            # Create UI elements.. how to not program an interface haha
            #
            radio_A = cmds.rowLayout( nc = 3, parent = self.shadingTab)
            radio_B = cmds.rowLayout( nc = 3, parent = self.shadingTab)
            self.radio_shader = cmds.radioCollection('hmRadioShaderCollection')

            for x, element in enumerate(self.SHADERS_NAMES.items()):
                if x <= 2:
                    par_to = radio_A
                else:
                    par_to = radio_B

                cmds.radioButton(element[1],
                                 label=element[0],
                                 collection = self.radio_shader,
                                 cc = self.shader_selection_changed,
                                 select= True if shader_stored == element[1] else False,
                                 parent = par_to)

            # Set the parent back
            cmds.setParent('..')

            cmds.rowLayout( nc = 2)


            # Create a drop down menu with the current scene shaders
            self.custom_shader_list = cmds.optionMenu(  label = labels['custom_shader'],
                                                        en = True if shader_stored == 3 else False,
                                                        changeCommand = self.set_custom_shader_name )

            # Fill the menu with items
            shaders = cmds.ls(mat=True)
            # print shaders
            for def_sh in ['lambert1', 'displacementShader1', 'particleCloud1']:
                try:
                    shaders.remove(def_sh)
                except:
                    pass
            for shader in shaders: cmds.menuItem( label = shader )

            # Select the stored value, if present
            if self.current_custom_shader:
                cmds.optionMenu( self.custom_shader_list, e=True, value = self.current_custom_shader )

            cmds.setParent('..')


        # Set the parent back to...	before enough...
        cmds.setParent('..')
        cmds.setParent('..')
        cmds.setParent('..')
        cmds.setParent('..')

    # ===========================================================================================================
    # ================================ COMMANDS ========================================================
    # ===========================================================================================================

    def performCommand(self, *Args ):
        """
        Perform the command
        """

        # Pass the current instance to the function that contains the logic for validating
        # and passing data to the node connector
        command.perform_new_operation(self)


    def saveOptionsValues(self):
        """
        This method stores the current options in the optionVar
        """
        # Get the interface values
        self.getValuesFromInterface()
        # Save them
        uiUtil.optionVarSave( self.optionsList )


    def resetOptionsValues(self):
        """
        This method reset the options with the defualt values
        """
        # Very important TRICK, let the gradient control forget about its option
        cmds.gradientControlNoAttr( self.gcProfileCurve, e=True, optionVar='' )

        # Reset the ui values
        if self.force_operation == 0:
            cmds.optionMenu(self.omOperation, e=True, select=self.hmopt_default_operation)

        cmds.intFieldGrp(self.ifgSmoothAB, e=True, value2= self.hmopt_default_smooth_b)
        cmds.intFieldGrp(self.ifgSmoothAB, e=True, value1= self.hmopt_default_smooth_a)

        if self.as_parameter_changer == True:
            cmds.floatSliderGrp(self.fsOffsetA, e=True, v= self.hmopt_default_offset_val)
            cmds.floatSliderGrp(self.fsOffsetB, e=True, v= self.hmopt_default_offset_val)
        else:
            cmds.floatSliderGrp(self.fsOffset, e=True, v= self.hmopt_default_offset_val)

        cmds.floatSliderGrp(self.fsGlobCorBias, e=True, v= self.hmopt_default_corner_bias)
        cmds.floatSliderGrp(self.fsGlobGeoBias, e=True, v= self.hmopt_default_geo_bias)
        cmds.intSliderGrp(self.isGlobSections, e=True, v= self.hmopt_default_sections)

        # Advanced
        # cmds.checkBox( self.cbRefEdge, e=True, v=self.hmopt_default_refine )
        cmds.optionMenuGrp(self.omRefEdge, e=True, select=self.hmopt_default_refine)


        cmds.floatSliderGrp(self.isAdaptVal, e=True, v=self.hmopt_default_chord_dist)


        cmds.checkBox(self.cbInterpProfile, e=True, v = self.hmopt_default_smooth_profile)
        # cmds.checkBox( self.cbRefFiller, e=True, v = self.hmOpt_defaultFillerRefinment  ) ##########
        cmds.floatSliderGrp(self.fsWeightA, e=True, v = self.hmopt_default_weight_a)
        cmds.floatSliderGrp(self.fsWeightB, e=True, v = self.hmopt_default_weight_b)
        cmds.checkBox(self.cbActiveRamp, e=True, v =self.hmopt_default_active_ramped_filler)
        # Hide the ramp controls, since the default has the use ramp unactive
        self.disableProfileControl()

        # Set the gradient control to the preset n.1
        self.gFunct.setGradientControlTo1

        cmds.floatSliderGrp(self.fsRampChord, e=True, v=self.hmopt_default_ramp_chord)
        cmds.floatSliderGrp(self.fsRampOffset, e=True, v=self.hmopt_default_ramp_offset)
        cmds.floatSliderGrp(self.fsRampScale, e=True, v=self.hmopt_default_ramp_scale)

        cmds.optionMenu(self.omInterp, e=True, select=self.hmopt_default_interpolation)
        if gVar.develop == 1:
            # == Order To Split
            # cmds.floatField( self.ffBindBias, edit=True, v=self.hmopt_default_bind_bias)
            # cmds.checkBox( self.cbSequencer, edit=True, v =self.hmOpt_defaultSequencer )
            # cmds.floatField( self.ffSeqTol, edit=True, v=self.hmopt_default_sequencer_tolerance)
            cmds.checkBox(self.cbRemCurl, edit=True, v=self.hmopt_default_remove_curl)

            # == Again compound
            # cmds.checkBox( self.cbAutoMerge, e=True, v = self.hmOpt_defaultAutoMerge )
            # cmds.floatField( self.ffGlobTol, e=True, v= self.hmOpt_defaultTolerance )
            cmds.checkBox(self.cbComputeNorm, e=True, v = self.hmopt_default_compute_normals)
            cmds.checkBox(self.cbRampMode, e=True, v= self.hmopt_default_ramp_mode)

        if not self.as_parameter_changer:
            # Shader
            cmds.radioCollection(self.radio_shader, edit=True, select=self.hmopt_default_output_shader)



    def connectCompoundNode(self, inCompoundNode, inAuxNode, meshes_list):
        """
        This method queries the compound attributes, and connect them to current node
        """

        # Set the current compound for having right attributes names
        self.connected_compound =  inCompoundNode
        self.connected_order_to_split = inAuxNode
        self.connected_offset_node = inAuxNode

        # Initialize the node attributes names
        self.initCompoundAttr(meshes_list)

        # ==== Operation
        cmds.connectControl(self.omOperation, self.nodeOperationType)

        # Set the state of the smooth field
        # if self.meshA.get_object_type() == nc.hm_out_mesh:
        # if self.meshB.get_object_type() == nc.hm_out_mesh:


        # Connect the smooth only if we have the right mesh type
        if self.mesh_A.get_object_type() == nc.hm_in_mesh:
            # print 'A da ATTIVARE!'
            cmds.intFieldGrp( self.ifgSmoothAB, edit=True, en1 = True)
            cmds.intFieldGrp(self.ifgSmoothAB, e=True, value1= cmds.getAttr(self.smooth_node_A))
        else:
            # print 'A da disattivareeee!'
            cmds.intFieldGrp( self.ifgSmoothAB, edit=True, en1 = False)


        if self.meshes_B[0].get_object_type() == nc.hm_in_mesh:
            # print 'B da ATTIVARE!'
            cmds.intFieldGrp( self.ifgSmoothAB, edit=True, en2 = True)
            cmds.intFieldGrp(self.ifgSmoothAB, e=True, value2= cmds.getAttr(self.smooth_nodes_B[0]))
        else:
            # print 'B da disattivareeee!'
            cmds.intFieldGrp( self.ifgSmoothAB, edit=True, en2 = False)


        # ==== Basics
        # This update the UI and connect the right controls
        self.updateOffsetStatus()

        cmds.connectControl(self.fsGlobCorBias, self.nodeCornerBias)
        cmds.connectControl(self.fsGlobGeoBias, self.nodeGeometryBias)
        cmds.connectControl(self.isGlobSections, self.nodeSectionSpans)

        # ==== Split Refinment
        cmds.connectControl(self.omRefEdge, self.nodeRefineEdge, index=2) # We need to use the index (the real control) or it wont work...


        cmds.connectControl(self.isAdaptVal, self.nodeChordDist)

        # Activate or deactivate the chord dist control
        # self.setRefineOptStatus()


        # ===== Interpolation and weights
        cmds.connectControl(self.cbInterpProfile, self.nodeSmoothFiller)


        # Activate or deactivate the weights
        self.setWeightsStatus()


        cmds.connectControl(self.fsWeightA, self.nodeWeightA)
        cmds.connectControl(self.fsWeightB, self.nodeWeightB)


        # ====== Ramped profile
        cmds.connectControl(self.cbActiveRamp, self.nodeUseRamp)

        # If the ramp is active, we need to make the tab visible, else to turn it off..
        if cmds.getAttr( self.nodeUseRamp ) == 1:
            self.enableProfileControl()
        else:
            self.disableProfileControl()

        # Connect the ramp chord, offset and scale controls

        cmds.connectControl(self.fsRampChord, self.nodeRampChord)
        cmds.connectControl(self.fsRampOffset, self.nodeRampOffset)
        cmds.connectControl(self.fsRampScale, self.nodeRampScale)

        # ================================================
        # ========= Sobstitute the current gradient control with a connected gradient control
        try:
            cmds.deleteUI(self.gcProfileCurve, self.omInterp, self.ffInterpPos, self.ffInterpVal, self.editGradButton )
        except:
            cmds.error('error in gradient control creation')


        # connect the gradient control for editing
        self.gFunct.setNode(self.connected_compound)



        self.gcProfileCurve = cmds.gradientControl( at=self.nodeRampedFiller, h=90, w= 200, parent= self.gradientParent )
        self.editGradButton = cmds.button(label= '>', h=90, w= 20, c=self.editGradientAttributeInWindow, visible = self.as_parameter_changer, parent= self.gradientParent)

        # Create the auxiliary controls
        self.omInterp  = cmds.attrEnumOptionMenuGrp( label = '', w=60, columnWidth = [1 ,0], attribute=self.nodeRampedFiller_InterpONE, parent = self.gradientAttrParent)
        self.ffInterpPos  = cmds.attrFieldSliderGrp( label = '', w=80, columnWidth = [1 ,0], attribute=self.nodeRampedFiller_PosONE, parent = self.gradientAttrParent)
        self.ffInterpVal  = cmds.attrFieldSliderGrp( label = '', w=80, columnWidth = [1 ,0], attribute=self.nodeRampedFiller_ValONE, parent = self.gradientAttrParent)

        # Connect them
        cmds.connectControl( self.omInterp , self.nodeRampedFiller_InterpONE)
        cmds.connectControl( self.ffInterpPos, self.nodeRampedFiller_PosONE)
        cmds.connectControl( self.ffInterpVal, self.nodeRampedFiller_ValONE)

        # Tell the port about the controls, this will update the controls.. yhee.. fuck it
        cmds.gradientControl(self.gcProfileCurve, edit=True, sic = self.omInterp  )
        cmds.gradientControl(self.gcProfileCurve, edit=True, spc = self.ffInterpPos )
        cmds.gradientControl(self.gcProfileCurve, edit=True, scc = self.ffInterpVal )




        if gVar.develop == 1:
            # ================================================
            # Normals sliders, we just create 1-1 connection
            # cmds.connectControl(self.cbAutoMerge, self.nodeAutoMerge)
            # cmds.connectControl(self.ffGlobTol, self.nodeMergeDist)
            cmds.connectControl(self.cbRampMode, self.nodeRampMode)
            cmds.connectControl(self.cbComputeNorm, self.nodeComputeNormals)

            # ================================================
            # Order to Split Attributes
            if cmds.nodeType( inAuxNode ) == 'hmOrderToSplitNode':
                # cmds.connectControl(self.ffBindBias, self.nodeBindBias)
                # cmds.connectControl(self.cbSequencer, self.nodeCurveSequencer)
                # cmds.connectControl(self.ffSeqTol, self.nodeSequenceBias)
                cmds.connectControl(self.cbRemCurl, self.nodeRemoveCurl)

            # ================================================
            # Offset Curve Node Attributes
            if cmds.nodeType( inAuxNode ) == 'hmOffsetCurveNode':
                cmds.connectControl(self.cbRemCurl, self.nodeOffsetRemoveCurl)
                # cmds.connectControl(self.cbSequencer, self.nodeCurveSequencer)
                # cmds.connectControl(self.ffSeqTol, self.nodeSequenceBias)


    def set_custom_shader_name(self, *args):
        """
        Method called every time something changes in the text field
        """
        # self.current_custom_shader = cmds.textField(self.custom_shader_field, q=True, text=True)
        self.current_custom_shader = cmds.optionMenu(self.custom_shader_list, q=True, value=True)
        # print self.current_custom_shader

    # ===========================================================================================================
    # ================================ INTERFACE UPDATES ========================================================
    # ===========================================================================================================
    def shader_selection_changed(self, *args):
        """
        Called when a radio selection changes
        """
        selected = cmds.radioCollection(self.radio_shader, q=True, select=True )

        # Set the custom value
        #
        cmds.optionMenu(self.custom_shader_list, edit =True,
                        en = True if selected == self.SHADERS_NAMES.values()[5] else False)

    def updateOffsetStatus(self):
        """
        Display one or two offset sliders
        """

        # Define the visibility of the controls
        if self.offsetIsSplitted():
            # print 'Dovrei avere DUE slider'
            cmds.columnLayout(self.doubleOffsetLayout, edit = True, visible = True)
            cmds.rowLayout( self.singleOffsetLayout, edit = True, visible = False)
            # Connect the controls
            cmds.connectControl(self.fsOffsetA, self.nodeOffsetAttrNameA)
            cmds.connectControl(self.fsOffsetB, self.nodeOffsetAttrNameB)

        else:
            # print 'Dovrei avere UNO slider'
            cmds.columnLayout(self.doubleOffsetLayout, edit = True, visible = False)
            cmds.rowLayout( self.singleOffsetLayout, edit = True, visible = True)
            # Connect the controls
            cmds.connectControl(self.fsOffset, self.nodeOffsetAttrNameA)

    def updateScaleOnCompound(self, *Args):
        """
        Trick for connecting the same value to different maya nodes...
        """
        if self.as_parameter_changer:
            # if self.connectedCompound != None:
            mainVal = cmds.floatSliderGrp( self.fsScaleFactor, query = True, value = True )

            # Update the chord dist with the same value of the scale if we have the option selected
            # cmds.setAttr( self.nodeCompoundScaleFactor, mainVal ) SCALE ON COMPOUND NOT PRESENT ANYMORE

            # If the option is set on scale, we set the chord dist with the same value of scale
            if cmds.optionMenuGrp( self.omRefEdge, query=True, select=True ) == 4:
                cmds.setAttr( self.nodeChordDist, mainVal )

    def enableSeqTol(self, *Args ):
        cmds.floatField( self.ffSeqTol, edit=True, en=True )
    def disableSeqTol(self, *Args ):
        cmds.floatField( self.ffSeqTol, edit=True, en=False )


    def enableProfileControl(self, *Args):
        # Make the ramp controls visible
        cmds.frameLayout( self.gradFrame, edit=True, visible=True )

    # Disable the sections spans
    # cmds.intSliderGrp( self.isGlobSections,  edit=True, en=False)
    def disableProfileControl(self, *Args):
        # Make the ramp controls invisible
        cmds.frameLayout( self.gradFrame, edit=True, visible=False )

    # Enable the sections spans
    # cmds.intSliderGrp( self.isGlobSections,  edit=True, en=True)

    # ======================== Chord Dist on off ==========================
    def setRefineOptStatus(self, *Args):
        """
        Activate the chord distance only when refine is on Chord Dist
        """
        cmds.optionMenuGrp( self.omRefEdge, query=True, select=True )


    def enableRefineOpt(self, *Args ):
        cmds.floatSliderGrp( self.isAdaptVal, edit=True, en=True )
        cmds.floatSliderGrp( self.isAdaptVal, edit=True, vis=True )
    def disableRefineOpt(self, *Args ):
        cmds.floatSliderGrp( self.isAdaptVal, edit=True, en=False )
        cmds.floatSliderGrp( self.isAdaptVal, edit=True, vis=False )

    # ======================== Weight AB on off ==========================
    def setWeightsStatus(self, *Args):
        valOfInterpolation = cmds.checkBox( self.cbInterpProfile, q=True, v = True )

        if valOfInterpolation == 0:
            cmds.floatSliderGrp( self.fsWeightA, e=True, vis = False )
            cmds.floatSliderGrp( self.fsWeightB, e=True, vis = False )
        else:
            cmds.floatSliderGrp( self.fsWeightA, e=True, vis = True )
            cmds.floatSliderGrp( self.fsWeightB, e=True, vis = True )



    def controlRampStatus(self):
        """
        Since opening and closing the frame layout ignore the status of the control
        making it always visible, we have to check it again
        """
        # Get the current check box value...
        isOnOrOff = cmds.checkBox( self.cbActiveRamp, q=True, v = True )
        # But if its a par changer, get the attribute...
        if self.as_parameter_changer:
            isOnOrOff = cmds.getAttr( self.nodeUseRamp )

        # Now we know if when opening the layout the control must be visible or not
        if isOnOrOff:
            self.enableProfileControl()
        else:
            self.disableProfileControl()

    # ===========================================================================================================
    # ================================ PARAMETER CHANGER COMMANDS ========================================================
    # ===========================================================================================================
    def changeSmoothValues(self, *Args):
        """
        This method sets the smooth values of the compound node.
        Because seems to be very slow, we put it here.
        """
        # Get value from UI
        self.getValuesFromInterface()

        # Set the attribute to the connected meshes, only if they have the smooth value
        if self.mesh_A.get_object_type() == nc.hm_in_mesh:
            cmds.setAttr( '%s.%s' % (self.mesh_A.get_name(), gVar.smooth_attr_name), self.hmopt_smooth_val_a[1])
        for mesh in self.meshes_B:
            if mesh.get_object_type() == nc.hm_in_mesh:
                cmds.setAttr( '%s.%s' % (mesh.get_name(), gVar.smooth_attr_name), self.hmopt_smooth_val_b[1])


    def editGradientAttributeInWindow(self, *Args):
        """
        Edit the current connected attribute in the standard maya window
        """
        mel.eval( 'editRampAttribute %s' % self.nodeRampedFiller)

    # ===========================================================================================================
    # ================================ UTILITY ========================================================
    # ===========================================================================================================

    def offsetIsSplitted(self):
        """
        This method returns True or False if depending on the connection
        of the current offset node attribute offsetB to the offsetA
        """
        if self.connected_offset_node:
            connected = cmds.listConnections('%s.%s' % (self.connected_offset_node, gVar.offset_B), p=True)
            # if there is a connection
            if connected:
                if connected[0].split('.')[1] == gVar.offset_A:
                    return False
                else:
                    return True
            else:
                return True

    def splitOffset(self, *Args):
        """
        This is called for splitting the current offset
        """

        # Break the connection
        driver = '%s.%s' % (self.connected_offset_node, gVar.offset_A)
        driven = '%s.%s' % (self.connected_offset_node, gVar.offset_B)

        cmds.disconnectAttr(driver, driven)

        # Update UI
        self.updateOffsetStatus()



    def mergeOffset(self, *Args):
        """
        This method is called for merging back the two offset values
        """
        # Create the connection
        driver = '%s.%s' % (self.connected_offset_node, gVar.offset_A)
        driven = '%s.%s' % (self.connected_offset_node, gVar.offset_B)

        cmds.connectAttr(driver, driven)

        # Update UI
        self.updateOffsetStatus()

    def getValuesFromInterface(self, *Args):
        """
        This method store in the main variables the values of the interface
        """
        self.hmopt_operation_val[1] = cmds.optionMenu(self.omOperation, q=True, select=True)
        self.hmopt_smooth_val_a[1] =cmds.intFieldGrp(self.ifgSmoothAB, q=True, value1=True)
        self.hmopt_smooth_val_b[1] = cmds.intFieldGrp(self.ifgSmoothAB, q=True, value2=True)

        if self.as_parameter_changer == True:
            self.opt_offset_val_a = cmds.floatSliderGrp(self.fsOffsetA, q=True, v=True)
            self.opt_offset_val_b = cmds.floatSliderGrp(self.fsOffsetB, q=True, v=True)
        else:
            self.hmopt_offset_val[1] = cmds.floatSliderGrp(self.fsOffset, q=True, v=True)

        self.hmopt_corner_bias_val[1] = cmds.floatSliderGrp(self.fsGlobCorBias, q=True, v=True)
        self.hmopt_geo_bias_val[1] = cmds.floatSliderGrp(self.fsGlobGeoBias, q=True, v=True)
        self.hmopt_sections_val[1] = cmds.intSliderGrp(self.isGlobSections, q=True, v=True)

        # Advanced
        # self.hmopt_refine_val[1] = cmds.checkBox( self.cbRefEdge, query=True, v=True )
        self.hmopt_refine_val[1] = cmds.optionMenuGrp(self.omRefEdge, query=True, select=True)

        self.hmopt_chord_dist_val[1] = cmds.floatSliderGrp(self.isAdaptVal, query=True, v=True)

        self.hmopt_smooth_profile_val[1] = cmds.checkBox(self.cbInterpProfile, q=True, v=True)
        # self.hmOpt_FillerRefinmentVal[1] =  cmds.checkBox( self.cbRefFiller, q=True, v = True )
        self.hmopt_weight_a_val[1] = cmds.floatSliderGrp(self.fsWeightA, q=True, v=True)
        self.hmopt_weight_b_val[1] = cmds.floatSliderGrp(self.fsWeightB, q=True, v=True)
        self.hmopt_active_ramped_filler_val[1] = cmds.checkBox(self.cbActiveRamp, q=True, v=True)
        # Query the profile curve with no attr only if the case
        if self.as_parameter_changer == False:
            # cmds.gradientControlNoAttr( self.gcProfileCurve, q=True, optionVar=True ) This only returns the optionVar name
            self.hmopt_profile_curve_val[1] = cmds.optionVar(q=self.hmopt_profile_curve_val[0])
            self.hmopt_interpolation_val[1] = cmds.optionMenu(self.omInterp, q=True, select=True)

        self.hmopt_ramp_offset_val[1] = cmds.floatSliderGrp(self.fsRampOffset, q=True, v=True)
        self.hmopt_ramp_scale_val[1] = cmds.floatSliderGrp(self.fsRampScale, q=True, v=True)

        if not self.as_parameter_changer:
            self.hmopt_out_shader_val[1] = cmds.radioCollection(self.radio_shader, q=True, select=True)

        if gVar.develop == 1:
            # == Order To Split Parameters
            # self.hmopt_bind_bias_val[1] = cmds.floatField( self.ffBindBias, q=True, v=True)
            # self.hmOpt_SequencerVal[1] = cmds.checkBox( self.cbSequencer, query=True, v =True )
            # self.hmOpt_SequencerToleranceVal[1] = cmds.floatField( self.ffSeqTol, q=True, v=True)
            self.hmopt_remove_curl_val[1] = cmds.checkBox(self.cbRemCurl, q=True, v=True)

            # == Again compound
            # self.hmOpt_AutoMergeVal[1] = cmds.checkBox( self.cbAutoMerge, query=True, v =True )
            # self.hmOpt_ToleranceVal[1] = cmds.floatField( self.ffGlobTol, q=True, v=True)
            self.hmopt_ramp_chord_val[1] = cmds.floatSliderGrp(self.fsRampChord, q=True, v=True)
            self.hmopt_compute_normals_val[1] = cmds.checkBox(self.cbComputeNorm, q=True, v=True)
            self.hmopt_ramp_mode_val[1] = cmds.checkBox(self.cbRampMode, q=True, v=True)

        # ====== DEVELOPING VAR
        if gVar.develop == 1 and self.as_parameter_changer == False:
            self.hmopt_offset_type[1] = cmds.radioButtonGrp(self.offTypeOperation, q=True, select=True)

    def getMainTooltip(self):
        """
        This method return the main tooltip hint for adding it to the main button
        """
        return tooltip['Create New Operation']

