#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import maya.cmds as cmds
import maya.mel as mel
import hmScripts as root
import hmScripts.hm_global_variables as global_vars
import hmScripts.UI.HmWindow_BaseOption as base_win
from hmScripts.UI.HmWindow_Workflow import HardMeshDockUI
import hmScripts.UI.HmOptions_HardMeshOperation as oper_opt
from hmScripts.hm_global_variables import language

# the formula for discovering the variable where a menu is stored in MEL : "g" + View + Name + "Menu"

mesh_lib_root = os.path.join(  os.path.dirname ( os.path.dirname( root.__path__[0] ) ), 'meshLibrary' )
labels = language.values['hm_menu']
titles = language.values['win_titles']

def hmMenu():

    manuName = 'hmMenuBar'

    #Window classes initialization
    hm_operation = base_win.HmOptionWindow( titles['hm_operation'], oper_opt.HardMeshOperationOptionsUI() )

    if cmds.menu( manuName, exists=True):
        cmds.deleteUI( manuName )


    gMainWindow = mel.eval('$temp1=$gMainWindow')
    #Parent to this to add to editMesh menu
    #editMeshMenu = mel.eval('$temp2=$gMainEditMeshMenu')

    mainMenu = cmds.menu(manuName, p =  gMainWindow, to = True, l= labels['menu_name'] )
    cmds.menuItem(parent = mainMenu, divider=True, dividerLabel = labels['windows_sep'])

    cmds.menuItem(parent= mainMenu, l=labels['workflow_window'], c= HardMeshDockUI.open )
    cmds.menuItem(parent = mainMenu, divider=True, dividerLabel = labels['mesh_sep'])
    cmds.menuItem(parent = mainMenu, l=labels['new_operation'], c=hm_operation.performFromMenu )
    cmds.menuItem(parent = mainMenu,  l=labels['new_operation'], c= hm_operation.buildAndShowWindow, ob=True )

    # -------------------------------------------------- EXPERIMENTAL ------------------------------------------------ #
    if global_vars.RELEASE >= 2.5:
        # splitWin = splitWindow.HmSplitWindow()
        # blendWin = blendWindow.HmBlendWindow()
        # snapWin = snapWindow.HmSnapWindow()

        cmds.menuItem(parent = mainMenu, l='Intersection', c= 'cmds.warning( \'empty command\')'  )
        cmds.menuItem(parent = mainMenu, l='Intersection', c= 'cmds.warning( \'empty command\')', ob=True )

        cmds.menuItem(parent = mainMenu, l=labels['split_mesh'], c= 'cmds.warning( \'empty command\')'  )
        cmds.menuItem(parent = mainMenu,  l=labels['split_mesh'], c= 'cmds.warning( \'empty command\')' , ob=True )

        cmds.menuItem(parent = mainMenu, divider=True, dividerLabel = 'Mesh Components')
        cmds.menuItem(parent = mainMenu, l=labels['blend_mesh'], c= 'cmds.warning( \'empty command\')'  )
        cmds.menuItem(parent = mainMenu,  l=labels['blend_mesh'], c= 'cmds.warning( \'empty command\')' , ob=True )

        cmds.menuItem(parent = mainMenu, divider=True, dividerLabel = 'Bonus')

        cmds.menuItem(parent = mainMenu, l=labels['edit_edge'], c='cmds.warning( \'empty command\')',  )
        cmds.menuItem(parent = mainMenu, l=labels['edit_edge'], c='cmds.warning( \'empty command\')', ob=True )

def hmMayaIntegration():
    """
    calling this will merge the hm commands to maya default menus..
    """
    pass



